@@ -89,22 +89,24 @@ those generators with cache-less generator streams.
 
 =head1 FUNCTIONS
 
+all of these functions are available with a ucfirst name, since many clash with
+perl builtin names.
+
 =head2 Utility Operations
 
 =over 4
 
-=item C< x_xs >
-
-=item C< x_xs GENERATOR >
+=item x_xs C< [GENERATOR] >
 
 C< x_xs > is a convenience function that returns the head and tail of a passed
 in generator.  C< x_xs > uses C< $_ > without an argument.
 
 =cut
 
-    sub x_xs (;$) {my $xs = @_ ? $_[0] : $_; $xs->head, $xs->tail}
+    sub x_xs (;$) {(@_, $_)[0]->x_xs}
 
-=item C<<< seq LIST >>>
+
+=item seq C< LIST >
 
 forces immediate evaluation of the elements in C< LIST > and returns the list
 
@@ -113,7 +115,7 @@ forces immediate evaluation of the elements in C< LIST > and returns the list
     BEGIN {*seq = \&now}
 
 
-=item C<<< flip CODE >>>
+=item flip C< {CODE} >
 
 C< flip > converts C< CODE > into a function that takes it's arguments reversed
 
@@ -132,11 +134,9 @@ C< flip > converts C< CODE > into a function that takes it's arguments reversed
 
 =over 4
 
-=item C<<< map :: (a -> b) -> [a] -> [b] >>>
+=item Map C< {CODE} LIST >
 
-=item C< map CODE LIST >
-
-map f xs is the list obtained by applying f to each element of xs, i.e.,
+Map f xs is the list obtained by applying f to each element of xs, i.e.,
 
     map f [x1, x2, ..., xn] == [f x1, f x2, ..., f xn]
     map f [x1, x2, ...] == [f x1, f x2, ...]
@@ -169,7 +169,7 @@ and this:
     let map => sub (&@) {&gen(shift, &lazy)};
 
 
-=item C<<< filter :: (a -> Bool) -> [a] -> [a] >>>
+=item filter C< {CODE} LIST >
 
 filter, applied to a predicate and a list, returns the list of those elements
 that satisfy the predicate; i.e.,
@@ -181,7 +181,7 @@ that satisfy the predicate; i.e.,
     let filter => sub (&@) {&List::Gen::filter(shift, &lazy)};
 
 
-=item C<<< head :: [a] -> a >>>
+=item head C< GENERATOR >
 
 Extract the first element of a list, which must be non-empty.
 
@@ -190,7 +190,7 @@ Extract the first element of a list, which must be non-empty.
     let head => sub {$_[0]->head};
 
 
-=item C<<< last :: [a] -> a >>>
+=item Last C< GENERATOR >
 
 Extract the last element of a list, which must be finite and non-empty.
 
@@ -199,7 +199,7 @@ Extract the last element of a list, which must be finite and non-empty.
     let last => sub {$_[0]->last};
 
 
-=item C<<< tail :: [a] -> [a] >>>
+=item tail C< GENERATOR >
 
 Extract the elements after the head of a list, which must be non-empty.
 
@@ -208,7 +208,7 @@ Extract the elements after the head of a list, which must be non-empty.
     let tail => sub {$_[0]->tail};
 
 
-=item C<<< init :: [a] -> [a] >>>
+=item init C< GENERATOR >
 
 Return all the elements of a list except the last one. The list must be
 non-empty.
@@ -221,7 +221,7 @@ non-empty.
     };
 
 
-=item C<<< null :: [a] -> Bool >>>
+=item null C< GENERATOR >
 
 Test whether a list is empty.
 
@@ -230,20 +230,20 @@ Test whether a list is empty.
     let null => sub {$_[0]->apply->size == 0};
 
 
-=item C<<< length :: [a] -> Int >>>
+=item Length C< GENERATOR >
 
 =cut
 
     let length => sub {$_[0]->apply->size};
 
 
-=item C<<< reverse :: [a] -> [a] >>>
+=item Reverse C< LIST >
 
 reverse xs returns the elements of xs in reverse order. xs must be finite.
 
 =cut
 
-    let reverse => sub {$_[0]->reverse};
+    let reverse => sub {&list->reverse};
 
 
 =back
@@ -252,7 +252,7 @@ reverse xs returns the elements of xs in reverse order. xs must be finite.
 
 =over 4
 
-=item C<<< foldl :: (a -> b -> a) -> a -> [b] -> a >>>
+=item foldl C< {CODE} ITEM LIST >
 
 foldl, applied to a binary operator, a starting value (typically the
 left-identity of the operator), and a list, reduces the list using the binary
@@ -262,7 +262,7 @@ operator, from left to right:
 
 The list must be finite.
 
-=item C<<< foldl1 :: (a -> a -> a) -> [a] -> a >>>
+=item foldl1 C< {CODE} LIST >
 
 C< foldl1 > is a variant of C< foldl > that has no starting value argument, and
 thus must be applied to non-empty lists.
@@ -294,7 +294,7 @@ thus must be applied to non-empty lists.
     };
 
 
-=item C<<< foldr :: (a -> b -> b) -> b -> [a] -> b >>>
+=item foldr C< {CODE} ITEM LIST >
 
 foldr, applied to a binary operator, a starting value (typically the
 right-identity of the operator), and a list, reduces the list using the binary
@@ -302,7 +302,7 @@ operator, from right to left:
 
     foldr f z [x1, x2, ..., xn] == x1 `f` (x2 `f` ... (xn `f` z)...)
 
-=item C<<< foldr1 :: (a -> a -> a) -> [a] -> a >>>
+=item foldr1 C< {CODE} LIST >
 
 C< foldr1 > is a variant of C< foldr > that has no starting value argument, and
 thus must be applied to non-empty lists.
@@ -336,7 +336,7 @@ thus must be applied to non-empty lists.
 
 =over 4
 
-=item C<<< and :: [Bool] -> Bool >>>
+=item And C< LIST >
 
 and returns the conjunction of a Boolean list. For the result to be True, the
 list must be finite; False, however, results from a False value at a finite
@@ -349,7 +349,7 @@ index of a finite or infinite list.
     };
 
 
-=item C<<< or :: [Bool] -> Bool >>>
+=item Or C< LIST >
 
 or returns the disjunction of a Boolean list. For the result to be False, the
 list must be finite; True, however, results from a True value at a finite index
@@ -362,7 +362,7 @@ of a finite or infinite list.
     };
 
 
-=item C<<< any :: (a -> Bool) -> [a] -> Bool >>>
+=item any C< {CODE} LIST >
 
 Applied to a predicate and a list, any determines if any element of the list
 satisfies the predicate.
@@ -375,7 +375,7 @@ satisfies the predicate.
     };
 
 
-=item C<<< all :: (a -> Bool) -> [a] -> Bool >>>
+=item all C< {CODE} LIST >
 
 Applied to a predicate and a list, all determines if all elements of the list
 satisfy the predicate.
@@ -388,7 +388,7 @@ satisfy the predicate.
     };
 
 
-=item C<<< sum :: Num a => [a] -> a >>>
+=item sum C< LIST >
 
 The sum function computes the sum of a finite list of numbers.
 
@@ -397,7 +397,7 @@ The sum function computes the sum of a finite list of numbers.
     let sum => sub {&lazy->sum};
 
 
-=item C<<< product :: Num a => [a] -> a >>>
+=item product C< LIST >
 
 The product function computes the product of a finite list of numbers.
 
@@ -406,7 +406,7 @@ The product function computes the product of a finite list of numbers.
     let product => sub {&lazy->product};
 
 
-=item C<<< concat :: [[a]] -> [a] >>>
+=item concat C< GENERATOR >
 
 Concatenate a list of lists.
 
@@ -431,7 +431,7 @@ Concatenate a list of lists.
     };
 
 
-=item C<<< concatMap :: (a -> [b]) -> [a] -> [b] >>>
+=item concatMap C< {CODE} LIST >
 
 Map a function over a list and concatenate the results.
 
@@ -440,7 +440,7 @@ Map a function over a list and concatenate the results.
     let concatMap => sub (&@) {concat(&gen(shift, &lazy))};
 
 
-=item C<<< maximum :: Ord a => [a] -> a >>>
+=item maximum C< LIST >
 
 maximum returns the maximum value from a list, which must be non-empty, finite,
 and of an ordered type. It is a special case of maximumBy, which allows the
@@ -451,7 +451,7 @@ programmer to supply their own comparison function.
     let maximum => sub {&lazy->max};
 
 
-=item C<<< minimum :: Ord a => [a] -> a >>>
+=item minimum C< LIST >
 
 minimum returns the minimum value from a list, which must be non-empty, finite,
 and of an ordered type. It is a special case of minimumBy, which allows the
@@ -470,7 +470,7 @@ programmer to supply their own comparison function.
 
 =over 4
 
-=item C<<< scanl :: (a -> b -> a) -> a -> [b] -> [a] >>>
+=item scanl C< {CODE} LIST >
 
 scanl is similar to foldl, but returns a list of successive reduced values from
 the left:
@@ -494,7 +494,7 @@ Note that
     };
 
 
-=item C<<< scanr :: (a -> b -> b) -> b -> [a] -> [b] >>>
+=item scanr C< {CODE} LIST >
 
 scanr is the right-to-left dual of scanl. Note that
 
@@ -519,7 +519,7 @@ scanr is the right-to-left dual of scanl. Note that
 
 =over 4
 
-=item C<<< iterate :: (a -> a) -> a -> [a] >>>
+=item iterate C< {CODE} ITEM >
 
 iterate f x returns an infinite list of repeated applications of f to x:
 
@@ -533,7 +533,7 @@ iterate f x returns an infinite list of repeated applications of f to x:
     };
 
 
-=item C<<< repeat :: a -> [a] >>>
+=item repeat C< ITEM >
 
 repeat x is an infinite list, with x the value of every element.
 
@@ -545,7 +545,7 @@ repeat x is an infinite list, with x the value of every element.
     };
 
 
-=item C<<< hs_repeat :: a -> [a] >>>
+=item hs_repeat C< ITEM >
 
     my $repeat; $repeat = lazy $x, $repeat;
 
@@ -557,7 +557,7 @@ repeat x is an infinite list, with x the value of every element.
     };
 
 
-=item C<<< replicate :: Int -> a -> [a] >>>
+=item replicate C< NUM ITEM >
 
 replicate n x is a list of length n with x the value of every element.
 
@@ -573,7 +573,7 @@ replicate n x is a list of length n with x the value of every element.
     };
 
 
-=item C<<< cycle :: [a] -> [a] >>>
+=item cycle C< LIST >
 
 cycle ties a finite list into a circular one, or equivalently, the infinite
 repetition of the original list. It is the identity on infinite lists.
@@ -583,7 +583,7 @@ repetition of the original list. It is the identity on infinite lists.
     let cycle => sub {&lazy->cycle};
 
 
-=item C<<< hs_cycle :: [a] -> [a] >>>
+=item hs_cycle C< LIST >
 
 hs_cycle ties a finite list into a circular one, or equivalently, the infinite
 repetition of the original list. It is the identity on infinite lists.
@@ -606,7 +606,7 @@ it is defined in perl as:
 
 =over 4
 
-=item C<<< take :: Int -> [a] -> [a] >>>
+=item take C< NUM LIST >
 
 take n, applied to a list xs, returns the prefix of xs of length n, or xs itself
 if n > length xs:
@@ -625,7 +625,7 @@ if n > length xs:
     };
 
 
-=item C<<< drop :: Int -> [a] -> [a] >>>
+=item drop C< NUM LIST >
 
 drop n xs returns the suffix of xs after the first n elements, or [] if n >
 length xs:
@@ -645,7 +645,7 @@ length xs:
     };
 
 
-=item C<<< splitAt :: Int -> [a] -> ([a], [a]) >>>
+=item splitAt C< NUM LIST >
 
 splitAt n xs returns a tuple where first element is xs prefix of length n and
 second element is the remainder of the list:
@@ -664,7 +664,7 @@ It is equivalent to (take n xs, drop n xs).
    let splitAt => sub {take(@_), drop(@_)}, 2, 2;
 
 
-=item C<<< takeWhile :: (a -> Bool) -> [a] -> [a] >>>
+=item takeWhile C< {CODE} LIST >
 
 takeWhile, applied to a predicate p and a list xs, returns the longest prefix
 (possibly empty) of xs of elements that satisfy p:
@@ -681,7 +681,7 @@ takeWhile, applied to a predicate p and a list xs, returns the longest prefix
     };
 
 
-=item C<<< dropWhile :: (a -> Bool) -> [a] -> [a] >>>
+=item dropWhile C< {CODE} LIST >
 
 dropWhile p xs returns the suffix remaining after take_while p xs:
 
@@ -697,7 +697,7 @@ dropWhile p xs returns the suffix remaining after take_while p xs:
     };
 
 
-=item C<<< span :: (a -> Bool) -> [a] -> ([a], [a]) >>>
+=item span C< {CODE} LIST >
 
 span, applied to a predicate p and a list xs, returns a tuple where first
 element is longest prefix (possibly empty) of xs of elements that satisfy p and
@@ -717,7 +717,7 @@ span p xs is equivalent to (takeWhile p xs, dropWhile p xs)
     }, 2, 2;
 
 
-=item C<<< break :: (a -> Bool) -> [a] -> ([a], [a]) >>>
+=item break C< {CODE} LIST >
 
 break, applied to a predicate p and a list xs, returns a tuple where first
 element is longest prefix (possibly empty) of xs of elements that do not satisfy
@@ -743,7 +743,7 @@ break p is equivalent to span (not . p)
 
 =over 4
 
-=item C<<< elem :: Eq a => a -> [a] -> Bool >>>
+=item elem C< ITEM LIST >
 
 elem is the list membership predicate, usually written in infix form, e.g.,
 x `elem` xs.
@@ -757,7 +757,7 @@ x `elem` xs.
     };
 
 
-=item C<<< notElem :: Eq a => a -> [a] -> Bool >>>
+=item notElem C< ITEM LIST >
 
 notElem is the negation of elem.
 
@@ -777,7 +777,7 @@ notElem is the negation of elem.
 
 =over 4
 
-=item C<<< zip :: [a] -> [b] -> [(a, b)] >>>
+=item zip C< LIST >
 
 zip takes 2+ lists and returns a single interleaved list. If one input list is
 short, excess elements of the longer lists are discarded. unlike the haskell
@@ -792,7 +792,7 @@ C<zip> is the same as C<zipWith {\@_}>
     };
 
 
-=item C<<< zipWith :: (a -> b -> c) -> [a] -> [b] -> [c] >>>
+=item zipWith C< {CODE} LIST >
 
 zipWith generalizes zip by zipping with the function given as the first
 argument, instead of a tupling function. For example, zipWith (+) is applied to
@@ -803,7 +803,7 @@ two lists to produce the list of corresponding sums.
     let zipWith => \&List::Gen::zipwith, 3;
 
 
-=item C<<< zipWithAB {$a * $b} $gen1, $gen2 >>>
+=item zipWithAB C< {$a * $b} $gen1, $gen2 >
 
 The zipWithAB function takes a function which uses C< $a > and C< $b >, as well
 as two lists and returns a list analogous to zipWith.
@@ -813,7 +813,7 @@ as two lists and returns a list analogous to zipWith.
     let zipWithAB => \&List::Gen::zipWithAB, 3;
 
 
-=item C<<< unzip :: [a, b] -> ([a], [b]) >>>
+=item unzip C< GENERATOR >
 
 unzip transforms a list into two lists of the even and odd elements.
 
@@ -828,7 +828,7 @@ unzip transforms a list into two lists of the even and odd elements.
     }
 
 
-=item C<<< unzipn Int, ... >>>
+=item unzipn C< NUM GENERATOR >
 
 The unzipn function is the n-dimentional precursor to C< unzip >
 
@@ -855,7 +855,7 @@ The unzipn function is the n-dimentional precursor to C< unzip >
 
 =over 4
 
-=item C<<< lines :: String -> [String] >>>
+=item lines C< STRING >
 
 lines breaks a string up into a list of strings at newline characters. The
 resulting strings do not contain newlines.  the newline sequence is taken from
@@ -874,7 +874,7 @@ the value of the input record separator C< $/ >
     };
 
 
-=item C<<< words :: String -> [String] >>>
+=item words C< STRING >
 
 words breaks a string up into a list of words, which were delimited by white
 space.
@@ -891,7 +891,7 @@ space.
     };
 
 
-=item C<<< unlines :: [String] -> String >>>
+=item unlines C< LIST >
 
 unlines is an inverse operation to lines. It joins lines, after appending a
 terminating newline to each. the newline sequence is taken from the value of the
@@ -902,7 +902,7 @@ input record separator C< $/ >
     let unlines => sub {&foldl1(sub {join $/ => @_}, &lazy)};
 
 
-=item C<<< unwords :: [String] -> String >>>
+=item unwords C< LIST >
 
 unwords is an inverse operation to words. It joins words with separating spaces.
 
